import { useState, useCallback, useEffect } from 'react';
import { selectAll, strictEqual } from './utils';
export var useEntity = function useEntity(entity, selector, equalityFn) {
  if (selector === void 0) {
    selector = selectAll;
  }

  if (equalityFn === void 0) {
    equalityFn = strictEqual;
  }

  var selected = selector(entity.state);

  var _useState = useState(selected),
      state = _useState[0],
      setState = _useState[1];

  var subscriberFn = useCallback(function (newState) {
    var newSelected = selector(newState);
    var hasChanged = !equalityFn(state, newSelected);
    if (hasChanged) setState(newSelected);
  }, [selector, equalityFn]);
  useEffect(function () {
    entity.subscribers.push(subscriberFn);
    return function () {
      for (var i = 0, c = entity.subscribers.length; i < c; i++) {
        if (entity.subscribers[i] === subscriberFn) {
          // Avoid causing subscribers array items to shift at this point.
          // The subscriber invocation loop in entity (see makeEntity.js)
          // should do the cleanup instead.
          // Was: entity.subscribers.splice(i, 1);
          entity.subscribers[i] = null;
          break;
        }
      }
    };
  }, [subscriberFn]);
  return [selected, entity.actions];
};
export default useEntity;